--[[=============================================================================
    Commands received from the camera proxy (ReceivedFromProxy)

    Copyright 2016 Control4 Corporation. All Rights Reserved.
===============================================================================]]

-- This macro is utilized to identify the version string of the driver template version used.
g_doPtz = 0
g_PTZtimer = -1
g_Ptzspeed = 5
g_PtzTimerSecs = 2
--1:Main Stream, 2:Sub Stream, 3:Third Stream
g_StreamType = 1

if (TEMPLATE_VERSION ~= nil) then
	TEMPLATE_VERSION.proxy_commands = "2016.01.08"
end

--[[
	Implement the following commands as required by capabilities.
--]]
function BuildCmd( subcmd, ... )  
     local command = "LAPI/V1.0/Channel/0/PTZ/"..subcmd
	local data = ""
	if( select('#', ...) > 0  ) then
	   local subcmdvalue = select(1, ...)
	    if( subcmd == "PTZCtrl") then 
		   local para1= select(2, ...)
		   local para2 = select(3, ...)
		   local para3 = select(4, ...)
		   data ="{\"PTZCmd\":"..subcmdvalue..",\"Para1\":".. para1..",\"Para2\":"..para2..",\"Para3\":"..para3.."}"
	   elseif(subcmd == "Presets") then
		   data = command + "/" + subcmdvalue.."/Goto"
	    end
     end
	
	return  command, data
end

function PutPTZCmdToDevice( inputdata, para1, para2, para3 )
	local command,data = BuildCmd("PTZCtrl",inputdata, para1, para2, para3 )
	local url = gCameraProxy:BuildHTTPURLWithoutPort(command)
	local ticket = gCon:UrlPut(data, gCameraProxy:BuildGetRequest(url), gCameraProxy:AuthHeader())
	LogTrace("Ticket = " .. ticket .. " URL = " .. url)
end

function PTZ_Control( inputdata, para1, para2, para3 )
     if( g_doPtz == 0 ) then
	    g_doPtz = 1
         PutPTZCmdToDevice( inputdata, para1, para2, para3 )
	    g_PTZtimer = C4:AddTimer(g_PtzTimerSecs, "SECONDS")
     end 
end

function PAN_LEFT()
	-- TODO: Set the value of the PAN_LEFT command
	LogTrace("PAN_LEFT():")
	PTZ_Control( 1284, g_Ptzspeed, g_Ptzspeed, 0 )
end

function PAN_RIGHT()
     -- TODO: Set the value of the PAN_LEFT command    
	LogTrace("PAN_RIGHT():")
     PTZ_Control( 1282, g_Ptzspeed, g_Ptzspeed, 0 )
end

function PAN_SCAN()
	-- TODO: Set the value of the PAN_SCAN command
	--Not Support
	LogTrace("PAN_SCAN():")
end

function TILT_UP()
	-- TODO: Set the value of the TILT_UP command
	LogTrace("TILT_UP():")
	PTZ_Control( 1026, 0, g_Ptzspeed, 0 )
end

function TILT_DOWN()
	-- TODO: Set the value of the TILT_DOWN command
	LogTrace("TILT_DOWN():")
     PTZ_Control( 1028, 0, g_Ptzspeed, 0 )
end

function TILT_SCAN()
	-- TODO: Set the value of the TILT_SCAN command
	--Not Support
	LogTrace("TILT_SCAN():")
end

function UPPER_RIGHT()
     -- TODO: Set the value of the PAN_LEFT command    
	LogTrace("UPPER_RIGHT():")
     PTZ_Control( 2050, g_Ptzspeed, g_Ptzspeed, 0 )
end

function LOWER_RIGHT()
     -- TODO: Set the value of the PAN_LEFT command    
	LogTrace("LOWER_RIGHT():")
     PTZ_Control( 2052, g_Ptzspeed, g_Ptzspeed, 0 )
end

function UPPER_LEFT()
     -- TODO: Set the value of the PAN_LEFT command    
	LogTrace("UPPER_LEFT():")
     PTZ_Control( 1794, g_Ptzspeed, g_Ptzspeed, 0 )
end

function LOWER_LEFT()
     -- TODO: Set the value of the PAN_LEFT command    
	LogTrace("LOWER_LEFT():")
     PTZ_Control( 1796, g_Ptzspeed, g_Ptzspeed, 0 )
end

function ZOOM_IN()
	-- TODO: Set the value of the ZOOM_IN command
	LogTrace("ZOOM_IN():")
	PTZ_Control( 770, 0, 0, 0 )
end

function ZOOM_OUT()
	-- TODO: Set the value of the ZOOM_OUT command
	LogTrace("ZOOM_OUT():")
	PTZ_Control( 772, 0, 0, 0 )
end

function PAN_STOP()
	PutPTZCmdToDevice( 2305, 0, 0, 0 )
end

function HOME()
	-- TODO: Set the value of the HOME command
	local command = "" 

	local url = gCameraProxy:BuildHTTPURL(command)
	local ticket = gCon:UrlGet(gCameraProxy:BuildGetRequest(url), gCameraProxy:AuthHeader())
	
	LogTrace("HOME():")
end

function MOVE_TO(width, height, x_index, y_index)
	-- TODO: Set the value of the MOVE_TO command
	x_index = (x_index - (width / 2)) / (width / 2)
	y_index = ((height / 2) - y_index) / (height/ 2)

	if ((x_index < 0) and (y_index < 0)) then
		LOWER_LEFT()
	elseif ((x_index > 0) and (y_index < 0)) then
		LOWER_RIGHT()
	elseif ((x_index > 0) and (y_index > 0)) then
		UPPER_RIGHT()
	elseif ((x_index < 0) and (y_index > 0)) then
		UPPER_LEFT()
	elseif ((x_index == 0) and (y_index > 0)) then
		TILT_UP()
     elseif ((x_index == 0) and (y_index < 0)) then
		TILT_DOWN()
	elseif ((x_index > 0) and (y_index == 0)) then
		PAN_RIGHT()
	elseif ((x_index < 0) and (y_index == 0)) then
		PAN_LEFT()
	end 
end

function PRESET(index)
	-- TODO: Set the value of the PRESET command
	local command,data = BuildCmd("Presets",index )
	local url = gCameraProxy:BuildHTTPURLWithoutPort(command)
	local ticket = gCon:UrlPut(data, gCameraProxy:BuildGetRequest(url), gCameraProxy:AuthHeader())
	
	LogTrace("PRESET(" .. tostring(index) .. "): Ticket = " .. ticket .. " URL = " .. url)
end

-- UI Requests
--[[
	Return the query string required for an HTTP snapshot URL request.
--]]
function GET_SNAPSHOT_QUERY_STRING(size_x, size_y)
	-- TODO: Add in the URL between the quotes.  Remember there should be no leading /
	-- If the camera has the ability to send a snapshot of a particular size then use the size_x and size_y variables in the query.
	local snapshotQueryString = "images/snapshot.jpg"

	return snapshotQueryString
end

--[[
	Return the query string required for an HTTP image push URL request.
--]]
function GET_MJPEG_QUERY_STRING(size_x, size_y, delay)
     --Not Support
	-- TODO: Add in the URL between the quotes.  Remember there should be no leading /
	-- If the camera has the ability to provide an MJPEG stream of a particular size then use the size_x and size_y variables in the query.
    local mjpegQueryString = "images/snapshot.jpg"

	return mjpegQueryString
end

--[[
	Return the query string required to establish Rtsp connection. May be empty string.
--]]
function GET_RTSP_H264_QUERY_STRING(size_x, size_y, delay)
	-- TODO: Add in the URL between the quotes.  Remember there should be no leading /
	-- If the camera has the ability to provide an H264 stream of a particular size then use the size_x and size_y variables in the query.
	local rtspH264QueryString = "media/video"..g_StreamType
	
	return rtspH264QueryString
end

function OnTimerExpired(idTimer)
    if idTimer == g_PTZtimer then
	   g_PTZtimer = C4:KillTimer(g_PTZtimer)
	   PAN_STOP()
	   g_doPtz = 0
    end
end